'use strict';

const request = require('request');
const debug = require('debug')('minvk');

module.exports = function(init) {
    if(typeof init !== "object")
        throw new Error("first argument must be an object");

    const user_data = {};
    if(init.mode === "access_token"){
        user_data.access_token = init.access_token;
        debug('Access token init')
    }
    else if(init.mode === "auth"){
        const uri = `https://oauth.vk.com/token?grant_type=password` +
                    `&client_id=2274003&client_secret=hHbZxrka2uZ6jB1inYsH` +
                    `&username=${init.username}&password=${init.password}`;
        request(uri, (err, res)=> {
            if(err){
                debug('auth error', err);
                throw new Error(JSON.stringify(err));
            }
            const data = JSON.parse(res.body);
            user_data.access_token = data.access_token;
            user_data.user_id = data.user_id;
            debug('auth success');
        });
        debug('Auth init');
    }
    else{
        debug('Wrong init mode');
        throw new Error("You should use 'access_token' or 'auth' as mode");
    }

    this.call = (name, param, callback)=> {
        debug('call', name, param);
        if(!callback) callback = () => 0;
        param.access_token = param.access_token || user_data.access_token;
        param.version = "5.52";
        request.post(`https://api.vk.com/method/${name}`, {form: param}, (err, res)=> {
            if(err){
                debug('call error', err);
                callback(err, null);
            }
            else{
                const data = JSON.parse(res.body);
                if(data.error){
                    debug('call error soft', data.error);
                    callback(true, data.error);
                }
                else{
                    debug('call success', data.response);
                    callback(null, data.response);
                }
            }
        })
    };

    this.longPoll = (callback)=> {
        debug('longpoll');
        if(!callback){
            debug('no callback, gtfo');
            return;
        }
        this.call("messages.getLongPollServer", {}, (err, res)=> {
            if(err){
                debug('getLongPollServer error', err);
                callback(err, res);
            }
            else{
                longPollHelper(res.server, res.key, res.ts, callback);
            }
        });
    };

    const longPollHelper = (server, key, ts, callback)=> {
        debug('longpollHelper ', server, key, ts);
        request(`https://${server}?act=a_check&key=${key}&ts=${ts}&wait=25&mode=2`, {timeout: 25000}, (err, res)=> {
            if(err){
                debug('longpollHelper error', err);
                this.longPoll(callback);
            }
            else{
                try{
                    const data = JSON.parse(res.body);
                    if(data.failed){
                        debug('longpollHelper error failed', data);
                        this.longPoll(callback);
                    }
                    else{
                        debug('got updates!', data.updates);
                        ts = data.ts;
                        callback(null, data.updates);
                        longPollHelper(server, key, ts, callback);
                    }
                }
                catch(e){
                    debug('longpollHelper error json', err);
                    this.longPoll(callback);
                }
            }

        });
    };

    this.messageGetAttachments = (id, callback)=> {
        debug('messageGetAttachments', id);
        this.call("messages.getById", {message_ids: id}, (err, res)=> {
            if(err){
                debug('messageGetAttachments err', err);
                callback(err, res);
            }
            else{
                debug('messageGetAttachments err', res[1].attachments);
                callback(err, res[1].attachments);
            }
        });
    };

    this.uploadPrivateImage = (stream, callback)=> {
        debug('uploadPrivateImage');
        this.call("photos.getMessagesUploadServer", {}, (err, res)=> {
            if(err || !res){
                debug('uploadprivateimage err', err);
                callback(err, res);
            }
            else{
                debug('uploadprivateimage url', res.upload_url);
                request.post(res.upload_url, {
                    formData: {
                        photo: stream
                    }
                }, (err, res) => {
                    const data = JSON.parse(res.body);
                    if(err || data.error){
                        debug('uploadprivateimage url err', err, data.error);
                        callback(true, data.error);
                    }
                    else{
                        debug('uploadprivateimage succ', data);
                        this.call("photos.saveMessagesPhoto", data, callback);
                    }
                });
            }
        });
    };

    this.isChatId = (id)=> id - 2e9 >= 0;

    this.sendMessageAuto = (text, id, att, callback)=> {
        const data = {
            message: text,
            attachment: att.join(',')
        };

        if(this.isChatId(id))
            data.chat_id = id - 2e9;
        else
            data.user_id = id;

        this.call("messages.send", data, callback);
    };
};